@@ -2,7 +2,9 @@ import threading
 from ctypes import POINTER, Structure, byref, c_char, c_char_p, c_int, c_size_t
 
 from django.contrib.gis.geos.base import GEOSBase
-from django.contrib.gis.geos.libgeos import GEOM_PTR, GEOSFuncFactory
+from django.contrib.gis.geos.libgeos import (
+    GEOM_PTR, GEOSFuncFactory, geos_version_info,
+)
 from django.contrib.gis.geos.prototypes.errcheck import (
     check_geom, check_sized_string, check_string,
 )
@@ -27,6 +29,7 @@ class WKBReader_st(Structure):
 class WKBWriter_st(Structure):
     pass
 
+
 WKT_READ_PTR = POINTER(WKTReader_st)
 WKT_WRITE_PTR = POINTER(WKTWriter_st)
 WKB_READ_PTR = POINTER(WKBReader_st)
@@ -101,6 +104,7 @@ class WKBWriterGet(GEOSFuncFactory):
 class WKBWriterSet(GEOSFuncFactory):
     argtypes = [WKB_WRITE_PTR, c_int]
 
+
 wkb_writer_get_byteorder = WKBWriterGet('GEOSWKBWriter_getByteOrder')
 wkb_writer_set_byteorder = WKBWriterSet('GEOSWKBWriter_setByteOrder')
 wkb_writer_get_outdim = WKBWriterGet('GEOSWKBWriter_getOutputDimension')
@@ -117,17 +121,10 @@ class IOBase(GEOSBase):
         self.ptr = self._constructor()
         # Loading the real destructor function at this point as doing it in
         # __del__ is too late (import error).
-        self._destructor.func = self._destructor.get_func(
-            *self._destructor.args, **self._destructor.kwargs
+        self.destructor.func = self.destructor.get_func(
+            *self.destructor.args, **self.destructor.kwargs
         )
 
-    def __del__(self):
-        # Cleaning up with the appropriate destructor.
-        try:
-            self._destructor(self._ptr)
-        except (AttributeError, TypeError):
-            pass  # Some part might already have been garbage collected
-
 # ### Base WKB/WKT Reading and Writing objects ###
 
 
@@ -136,8 +133,8 @@ class IOBase(GEOSBase):
 # objects.
 class _WKTReader(IOBase):
     _constructor = wkt_reader_create
-    _destructor = wkt_reader_destroy
     ptr_type = WKT_READ_PTR
+    destructor = wkt_reader_destroy
 
     def read(self, wkt):
         if not isinstance(wkt, (bytes, six.string_types)):
@@ -147,8 +144,8 @@ class _WKTReader(IOBase):
 
 class _WKBReader(IOBase):
     _constructor = wkb_reader_create
-    _destructor = wkb_reader_destroy
     ptr_type = WKB_READ_PTR
+    destructor = wkb_reader_destroy
 
     def read(self, wkb):
         "Returns a _pointer_ to C GEOS Geometry object from the given WKB."
@@ -164,8 +161,8 @@ class _WKBReader(IOBase):
 # ### WKB/WKT Writer Classes ###
 class WKTWriter(IOBase):
     _constructor = wkt_writer_create
-    _destructor = wkt_writer_destroy
     ptr_type = WKT_WRITE_PTR
+    destructor = wkt_writer_destroy
 
     _trim = False
     _precision = None
@@ -217,20 +214,44 @@ class WKTWriter(IOBase):
 
 class WKBWriter(IOBase):
     _constructor = wkb_writer_create
-    _destructor = wkb_writer_destroy
     ptr_type = WKB_WRITE_PTR
+    destructor = wkb_writer_destroy
 
     def __init__(self, dim=2):
         super(WKBWriter, self).__init__()
         self.outdim = dim
 
+    def _handle_empty_point(self, geom):
+        from django.contrib.gis.geos import Point
+        if isinstance(geom, Point) and geom.empty:
+            if self.srid:
+                # PostGIS uses POINT(NaN NaN) for WKB representation of empty
+                # points. Use it for EWKB as it's a PostGIS specific format.
+                # https://trac.osgeo.org/postgis/ticket/3181
+                geom = Point(float('NaN'), float('NaN'), srid=geom.srid)
+            else:
+                raise ValueError('Empty point is not representable in WKB.')
+        return geom
+
     def write(self, geom):
         "Returns the WKB representation of the given geometry."
-        return six.memoryview(wkb_writer_write(self.ptr, geom.ptr, byref(c_size_t())))
+        from django.contrib.gis.geos import Polygon
+        geom = self._handle_empty_point(geom)
+        wkb = wkb_writer_write(self.ptr, geom.ptr, byref(c_size_t()))
+        if geos_version_info()['version'] < '3.6.1' and isinstance(geom, Polygon) and geom.empty:
+            # Fix GEOS output for empty polygon.
+            # See https://trac.osgeo.org/geos/ticket/680.
+            wkb = wkb[:-8] + b'\0' * 4
+        return six.memoryview(wkb)
 
     def write_hex(self, geom):
         "Returns the HEXEWKB representation of the given geometry."
-        return wkb_writer_write_hex(self.ptr, geom.ptr, byref(c_size_t()))
+        from django.contrib.gis.geos.polygon import Polygon
+        geom = self._handle_empty_point(geom)
+        wkb = wkb_writer_write_hex(self.ptr, geom.ptr, byref(c_size_t()))
+        if geos_version_info()['version'] < '3.6.1' and isinstance(geom, Polygon) and geom.empty:
+            wkb = wkb[:-16] + b'0' * 8
+        return wkb
 
     # ### WKBWriter Properties ###
 
@@ -281,6 +302,7 @@ class ThreadLocalIO(threading.local):
     wkb_w = None
     ewkb_w = None
 
+
 thread_context = ThreadLocalIO()
 
 
